@@ -26,7 +26,7 @@ class ContainerLauncher:
     __ECS_TASK_EXECUTION_POLICY_ARN = 'arn:aws:iam::aws:policy/service-role/AmazonECSTaskExecutionRolePolicy'
     __IAM_EC2_MANAGED_POLICY_ARNS = [
         'arn:aws:iam::aws:policy/service-role/AmazonEC2ContainerServiceforEC2Role',
-        #'arn:aws:iam::aws:policy/AmazonSSMManagedInstanceCore'
+        # 'arn:aws:iam::aws:policy/AmazonSSMManagedInstanceCore'
     ]
 
     def __init__(self, options=None):
@@ -84,7 +84,8 @@ class ContainerLauncher:
             logging.basicConfig(level=log_level)
 
         if options is not None and 'spot_instance_deploying_timeout_in_minutes' in options:
-            self.__SPOT_INSTANCE_DEPLOYING_TIMEOUT_IN_MINUTES = options.get('spot_instance_deploying_timeout_in_minutes')
+            self.__SPOT_INSTANCE_DEPLOYING_TIMEOUT_IN_MINUTES = options.get(
+                'spot_instance_deploying_timeout_in_minutes')
 
     def start_container(self, container_input):
         entry_id = container_input.get('id', str(uuid.uuid4()).replace('-', ''))
@@ -462,13 +463,15 @@ class ContainerLauncher:
                     PolicyArn=managed_policy_arn
                 )
             ec2_iam_role_policy_document_names = self.__resources['ec2_iam_role_policy_document_names']
-            if self.__resources.get('ec2_iam_role_name') and ec2_iam_role_policy_document_names and len(ec2_iam_role_policy_document_names) > 0:
+            if self.__resources.get('ec2_iam_role_name') and ec2_iam_role_policy_document_names and len(
+                    ec2_iam_role_policy_document_names) > 0:
                 for ec2_iam_role_policy_document_name in ec2_iam_role_policy_document_names:
                     self.clients['iam'].delete_role_policy(
                         RoleName=self.__resources['ec2_iam_role_name'],
                         PolicyName=ec2_iam_role_policy_document_name
                     )
-                self.logger.debug(f"The IAM policies {str(self.__resources['ec2_iam_role_policy_document_names'])} has been deleted")
+                self.logger.debug(
+                    f"The IAM policies {str(self.__resources['ec2_iam_role_policy_document_names'])} has been deleted")
         except Exception as e:
             self.logger.warning(f'An error occurred during the delete_role_policy operation: {str(e)}')
         try:
@@ -476,7 +479,8 @@ class ContainerLauncher:
                 self.clients['iam'].delete_role(RoleName=self.__resources['ec2_iam_role_name'])
                 self.logger.debug(f"EC2 IAM role {self.__resources['ec2_iam_role_name']} has been deleted")
         except Exception as e:
-            self.logger.warning(f"An error occurred during the {self.__resources['ec2_iam_role_name']} delete_role operation: {str(e)}")
+            self.logger.warning(
+                f"An error occurred during the {self.__resources['ec2_iam_role_name']} delete_role operation: {str(e)}")
         try:
             if self.__resources.get('instance_profile_name'):
                 self.clients['iam'].delete_instance_profile(
@@ -579,7 +583,8 @@ class ContainerLauncher:
                 self.logger.debug(
                     f"The ECS task IAM role {self.__resources.get('ecs_task_iam_role_name')} has been deleted")
         except Exception as e:
-            self.logger.warning(f"An error occurred during the {self.__resources.get('ecs_task_iam_role_name')} delete_role operation: {str(e)}")
+            self.logger.warning(
+                f"An error occurred during the {self.__resources.get('ecs_task_iam_role_name')} delete_role operation: {str(e)}")
         try:
             ecs_task_execution_iam_role_name = self.__resources.get('ecs_task_execution_iam_role_name')
             if ecs_task_execution_iam_role_name:
@@ -591,7 +596,8 @@ class ContainerLauncher:
                 self.logger.debug(
                     f"The ECS task execution IAM role {ecs_task_execution_iam_role_name} has been deleted")
         except Exception as e:
-            self.logger.warning(f"An error occurred during the {self.__resources.get('ecs_task_execution_iam_role_name')} delete_role operation: {str(e)}")
+            self.logger.warning(
+                f"An error occurred during the {self.__resources.get('ecs_task_execution_iam_role_name')} delete_role operation: {str(e)}")
 
     def __create_asg(self, asg_opts):
         name_suffix = self.__options['name_suffix']
@@ -1073,6 +1079,22 @@ mount /dev/md0 %s
                 mount_name = file_system['LustreConfiguration']['MountName']
                 install_fsx_agent_cmd = 'amazon-linux-extras install -y lustre2.10'
                 user_data = f'{user_data}\n{install_fsx_agent_cmd}\nmkdir {fsx_dir}\nmount -t lustre -o noatime,flock {dns_name}@tcp:/{mount_name} {fsx_dir}'
+        block_devices = None
+        if storages is not None:
+            ebs_storages = [storage for storage in storages if storage['type'] == 'ebs']
+            block_devices = []
+            for ebs_storage in ebs_storages:
+                block_device = {
+                    'DeviceName': ebs_storage['device_name'],
+                    'Ebs': {
+                        'Encrypted': False,
+                        'DeleteOnTermination': True,
+                        'VolumeSize': ebs_storage['volume_size'],
+                        'VolumeType': ebs_storage['volume_type']  # 'standard'|'io1'|'io2'|'gp2'|'sc1'|'st1'|'gp3',
+                    }
+                }
+                block_devices.append(block_device)
+
         output_logs_command = 'exec > >(tee /var/log/user-data.log|logger -t user-data -s 2>/dev/console) 2>&1'
         user_data = f'#!/bin/bash\n{install_ecs_agent_cmd}\n{output_logs_command}\n{user_data}'
 
@@ -1093,9 +1115,12 @@ mount /dev/md0 %s
                     'ResourceType': 'volume',
                     'Tags': self.__tags
                 },
-            ],
+            ]
         }
 
+        if block_devices is not None and len(block_devices) > 0:
+            launch_template_data['BlockDeviceMappings'] = block_devices
+
         if ec2_sg_ids is not None and len(ec2_sg_ids) > 0:
             launch_template_data['SecurityGroupIds'] = ec2_sg_ids
 
